#include "heat_equation.h"


//       ,      .
__device__ float f_gpu(float x, float y)
{
    return 0.05f * x * x + 0.001f * y * y * y;
}


// newV(i, j) = f(x, y) + v(i, j) +
//    dt * ((v(i + 1, j) - 2 * v(i, j) + v(i - 1, j)) / dx^2 +
//          (v(i, j + 1) - 2 * v(i, j) + v(i, j - 1)) / dy^2),
//  i = 1, 2, ..., nx - 1, j = 1, 2, ..., ny -  1,
__global__ void kernel(int nx, int ny, float dx, float dy, float dt, float a,
    float c, const float * v, float * newV)
{
    //    i, j.
    int i = 1 + blockIdx.x * blockDim.x + threadIdx.x;
    int j = 1 + blockIdx.y * blockDim.y + threadIdx.y;

    //     .
    //       n,   .
    if ((i < nx - 1) && (j < ny - 1))
    {
        float x = a + i * dx;
        float y = c + j * dy;
        newV[i * ny + j] = v[i * ny + j] + dt * (f_gpu(x, y) + 
            (v[(i + 1) * ny + j] - 2.0f * v[i * ny + j] + v[(i - 1) * ny + j]) / (dx * dx) +
            (v[i * ny + (j + 1)] - 2.0f * v[i * ny + j] + v[i * ny + (j - 1)]) / (dy * dy));
    }
}



/*      
du/dt = d2u/dx^2 + d2u/dy^2 + f(x, t)
      .
nx, ny -      x, y,
dx, dy -     x, y,
a, c -     x, y,
v_gpu -  ,
newV -  ,   .
v_gpu, newV_gpu -     GPU.
      .
*/
void step_gpu(int nx, int ny, float dx, float dy, float dt, float a, float c,
    const float * v_gpu, float * newV_gpu)
{
    //         .
    const int threadsX = 16;
    const int threadsY = 16;
    dim3 blocks(threadsX, threadsY);
    dim3 grid(((nx - 2) + (threadsX - 1)) / threadsX,
              ((ny - 2) + (threadsY - 1)) / threadsY);

    //  .
   kernel<<<grid, blocks>>>(nx, ny, dx, dy, dt, a, c, v_gpu, newV_gpu);
}
